from bs4 import BeautifulSoup
from typing import List
from requests import get
from .types import Picture, TAG, CATALOG, RESOLUTIONS


class WallpapersCraftAPI:
    """Sync API for wallpaperscraft.com"""

    WEBSITE = "https://wallpaperscraft.com"
    
    def get(self, query: str = "/", **kwargs):
        if not query.startswith(self.WEBSITE):
            query = self.WEBSITE + query
        
        responce = get(query, **kwargs)
        if not responce.raise_for_status():
            return BeautifulSoup(responce.text)
    
    def get_all_pictures_from_page(self, page: BeautifulSoup):
        return [
            Picture(
                link=self.WEBSITE + pic.find("a", {"class": "wallpapers__link"}).get("href"),
                info=pic.find("a", {"class": "wallpapers__link"}).find_all("span", {"class": "wallpapers__info"})[1].text,
                downloads=pic.find("a", {"class": "wallpapers__link"}).find_all("span", {"class": "wallpapers__info"})[0].find("span", {"class": "wallpapers__info-downloads"}).text,
                rating=pic.find("a", {"class": "wallpapers__link"}).find_all("span", {"class": "wallpapers__info"})[0].find("span", {"class": "wallpapers__info-rating"}).text.strip(),
                api_class=self
            )
            for pic in page.find_all("li", {"class": "wallpapers__item"})
        ]

    def get_wallpaper(self, link: str):
        return self.get(link).find("img", {"class": "wallpaper__image"}).get("src")

    def all(self, page: int = 1, resolution: str = "") -> List[Picture]:
        """All pictures."""
        if resolution:
            if resolution not in RESOLUTIONS:
                raise ValueError("Parameter <resolution> isn't a valid screen resolution! Please set a valid resolution.")
            resolution = "/" + resolution
        
        return self.get_all_pictures_from_page(self.get(f"/all{resolution}/page{page}"))
    
    def get_by_tag(self, tag: TAG, page: int = 1, resolution: str = ""):
        """Get pictures by tag and page"""

        if type(tag) != TAG:
            raise TypeError("Parameter tag should be a value from wallpaperscraft.TAG")
        
        if resolution:
            if resolution not in RESOLUTIONS:
                raise ValueError("Parameter <resolution> isn't a valid screen resolution! Please set a valid resolution.")
            resolution = "/" + resolution
        
        return self.get_all_pictures_from_page(self.get(f"/tag/{tag.value}{resolution}/page{page}"))
    
    def get_by_catalog(self, catalog: CATALOG, page: int = 1, resolution: str = ""):
        """Get pictures by catalog and page"""

        if type(catalog) != CATALOG:
            raise TypeError("Parameter tag should be a value from wallpaperscraft.CATALOG")
        
        if resolution:
            if resolution not in RESOLUTIONS:
                raise ValueError("Parameter <resolution> isn't a valid screen resolution! Please set a valid resolution.")
            resolution = "/" + resolution
        
        return self.get_all_pictures_from_page(self.get(f"/catalog/{catalog.value}{resolution}/page{page}"))
    
    def search(self, query: str, page: int = 1, resolution: str = ""):
        """Get pictures by search query and page"""
        if resolution and resolution not in RESOLUTIONS:
            raise ValueError("Parameter <resolution> isn't a valid screen resolution! Please set a valid resolution.")

        return self.get_all_pictures_from_page(
            self.get(
                "https://wallpaperscraft.com/search/",
                params={
                    "order": "",
                    "page": str(page),
                    "query": query.strip().replace(" ", "+"),
                    "size": resolution
                }
            )
        )





from setuptools import setup, find_packages
import re


github = "https://github.com/Den4ikSuperOstryyPer4ik/WallpapersCraftAPI/"

def get_version():
    with open("wallpaperscraft/__init__.py", encoding="utf-8") as f:
        return re.findall(r"__version__ = \"(.+)\"", f.read())[0]

def get_readme():
    with open("README.md", encoding="utf-8") as f:
        return f.read()


setup(
    name="wallpaperscraft",
    fullname="WallpapersCraftAPI",
    version=get_version(),
    author="Den4ikSuperOstryyPer4ik",
    description="Unofficial API WallpapersCraft (wallpaperscraft.com)",
    long_description=get_readme(),
    long_description_content_type="text/markdown",
    url=github,
    packages=find_packages(),
    download_url=f"{github}/releases/latest",
    requires=[
        "bs4",
        "requests",
        "aiohttp",
        "pydantic",
    ],
    classifiers=[
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.9",
    ],
    keywords="python3 wallpaperscraft wallpapers craft WallpapersCraftAPI",
    python_requires='>=3.7',
    license="Apache2.0",
)



from typing import Optional, Any
from pydantic import BaseModel
from enum import Enum


RESOLUTIONS = [
    "1920x1080",
    "240x320",
    "240x400",
    "320x240",
    "320x480",
    "360x640",
    "480x800",
    "480x854",
    "540x960",
    "720x1280",
    "800x600",
    "800x1280",
    "960x544",
    "1024x600",
    "1080x1920",
    "2160x3840",
    "1366x768",
    "1440x2560",
    "800x1200",
    "800x1420",
    "938x1668",
    "1280x1280",
    "1350x2400",
    "3415x3415",
    "2780x2780",
    "1024x768",
    "1152x864",
    "1280x960",
    "1400x1050",
    "1600x1200",
    "1280x1024",
    "1280x720",
    "1280x800",
    "1440x900",
    "1680x1050",
    "1920x1200",
    "2560x1600",
    "1600x900",
    "2560x1440",
    "2048x1152",
    "2560x1024",
    "2560x1080",
    "3840x2400",
    "3840x2160",
]


class Picture(BaseModel):
    link: str
    info: str
    downloads: int
    rating: Optional[str] = None
    api_class: Any

    def download(self, resolution: str = "1024x768"):
        """
        Get link for picture by resolution
        """
        if resolution not in RESOLUTIONS:
            raise ValueError("Parameter <resolution> isn't a valid screen resolution! Please set a valid resolution.")
        URL = f"{self.link.replace('/wallpaper/', '/download/')}"
        if not any(URL.endswith(i) for i in RESOLUTIONS):
            URL += f"/{resolution}"
        return self.api_class.get_wallpaper(URL)


class CATALOG(Enum):
    _3D = "3d"
    ABSTRACT = "abstract"
    ANIMALS = "animals"
    ANIME = "anime"
    ART = "art"
    BLACK = "black"
    BLACK_AND_WHITE = "black_and_white"
    CARS = "cars"
    CITY = "city"
    DARK = "dark"
    FANTASY = "fantasy"
    FLOWERS = "flowers"
    FOOD = "food"
    HOLIDAYS = "holidays"
    LOVE = "love"
    MACRO = "macro"
    MINIMALISM = "minimalism"
    MOTORCYCLES = "motorcycles"
    MUSIC = "music"
    NATURE = "nature"
    OTHER = "other"
    SPACE = "space"
    SPORT = "sport"
    TECHNOLOGIES = "technologies"
    TEXTURES = "textures"
    VECTOR = "vector"
    WORDS = "words"


class TAG(Enum):
    ANGEL = "angel"
    ART = "art"
    ASTON_MARTIN = "aston martin"
    AUTUMN = "autumn"
    BLACK = "black"
    BLONDE = "blonde"
    BRUNETTE = "brunette"
    BUGATTI = "bugatti"
    CAT = "cat"
    DOG = "dog"
    DRAGON = "dragon"
    FERRARI = "ferrari"
    FUNNY = "funny"
    GALAXY = "galaxy"
    HEART = "heart"
    KITTEN = "kitten"
    LAMBORGHINI = "lamborghini"
    LION = "lion"
    LOVE = "love"
    MINIMALISM = "minimalism"
    OWL = "owl"
    PUPPY = "puppy"
    ROBOT = "robot"
    ROSE = "rose"
    SEA = "sea"
    SKY = "sky"
    SPRING = "spring"
    STARS = "stars"
    SUMMER = "summer"
    SUN = "sun"
    TIGER = "tiger"
    WHITE = "white"
    WINTER = "winter"
    WOLF = "wolf"
    ZOMBIE = "zombie"
